import cv2 as cv

from ..engine.base_node import Node


class EdgeDetection(Node):
    def __init__(self, **kwargs):
        super(EdgeDetection, self).__init__(**kwargs)

    def call(self, inputs, **kwargs):
        gray = cv.cvtColor(inputs, cv.COLOR_BGR2GRAY)
        laplacian = cv.Laplacian(gray, cv.CV_8U)
        output = cv.cvtColor(laplacian, cv.COLOR_GRAY2RGB)
        return output


class Sink(Node):
    def __init__(self, **kwargs):
        super(Sink, self).__init__(**kwargs)

    def call(self, inputs, **kwargs):
        return self


class AutoVideoSink(Sink):
    def __init__(self, **kwargs):
        super(AutoVideoSink, self).__init__(**kwargs)

    def call(self, inputs, **kwargs):
        cv.imshow('AutoVideoSink', inputs)
        return None


class FileSink(Sink):
    def __init__(self, **kwargs):
        super(FileSink, self).__init__(**kwargs)
        fourcc = cv.VideoWriter_fourcc(*'MPEG')
        self.out = cv.VideoWriter(filename='output.avi', fourcc=fourcc, fps=25.0, frameSize=(1280, 720), isColor=True)

    def call(self, inputs, **kwargs):
        self.out.write(inputs)
        return None



import re


class Node(object):

    def __init__(self, **kwargs):
        self.built = False
        self._inbound_nodes = []
        self._outbound_nodes = []

        allowed_kwargs = {'input_shape',
                          'batch_size',
                          'dtype',
                          'name',
                          }
        for kwarg in kwargs:
            if kwarg not in allowed_kwargs:
                raise TypeError('Keyword argument not understood:', kwarg)

        name = kwargs.get('name')
        if not name:
            prefix = self.__class__.__name__
            name = _to_snake_case(prefix) + '_' + str(1)
        self.name = name

    def call(self, inputs, **kwargs):
        return inputs

    def __call__(self, node, inputs=None, **kwargs):
        # print('__call__ inputs: ' + str(inputs))
        self._add_inbound_node(node)
        return self

    def build(self):
        self.built = True

    def _add_inbound_node(self, input_tensors):
        self._inbound_nodes.append(input_tensors)


def _to_snake_case(name):
    intermediate = re.sub('(.)([A-Z][a-z0-9]+)', r'\1_\2', name)
    insecure = re.sub('([a-z])([A-Z])', r'\1_\2', intermediate).lower()
    # If the class is private the name starts with "_" which is not secure
    # for creating scopes. We prefix the name with "private" in this case.
    if insecure[0] != '_':
        return insecure
    return 'private' + insecure



from .base_node import Node
import cv2 as cv


class SourceNode(Node):
    # Node to be used as an entry point into a pipeline.
    def __init__(self, name=None):
        if not name:
            prefix = 'source'
            name = prefix + '_' + str(1)
        super(SourceNode, self).__init__(name=name)

    def call(self, inputs=None, **kwargs):
        return None


class VideoTestSource(SourceNode):
    def __init__(self, name=None):
        super(VideoTestSource, self).__init__(name=name)
        self._cap = cv.VideoCapture('videotestsrc pattern=snow ! video/x-raw,width=1280,height=720 ! appsink sync=false ', cv.CAP_GSTREAMER)

    def call(self, inputs=None, **kwargs):
        return self._cap.read()


class NetworkSource(SourceNode):
    def __init__(self, name=None):
        super(NetworkSource, self).__init__(name=name)
        self._cap = cv.VideoCapture('udpsrc port=5000 ! application/x-rtp, payload=96 ! rtpjitterbuffer ! rtph264depay ! avdec_h264  ! videoconvert  ! queue ! appsink sync=false ', cv.CAP_GSTREAMER)

    def call(self, inputs=None, **kwargs):
        return self._cap.read()


class FileSource(SourceNode):
    def __init__(self, location):
        super(FileSource, self).__init__()
        command = 'filesrc location={} ! decodebin ! videoconvert ! queue ! appsink sync=false '.format(location)
        print(command)
        self._cap = cv.VideoCapture(command, cv.CAP_GSTREAMER)

    def call(self, inputs=None, **kwargs):
        return self._cap.read()



import os

import pydot


def graph_to_dot(graph,
                 show_shapes=False,
                 show_node_names=True):
    from ..nodes.wrappers import Wrapper
    from ..pipes import Sequential

    dot = pydot.Dot()
    dot.set('concentrate', True)
    dot.set_node_defaults(shape='record')

    if isinstance(graph, Sequential):
        if not graph.built:
            graph.build()
    nodes = graph._nodes
    print('len(nodes): ' + str(len(nodes)))

    # Create graph nodes.
    for node in nodes:
        node_id = str(id(node))

        # Append a wrapped layer's label to node's label, if it exists.
        node_name = node.name
        class_name = node.__class__.__name__
        if isinstance(node, Wrapper):
            node_name = '{}({})'.format(node_name, node.layer.name)
            child_class_name = node.layer.__class__.__name__
            class_name = '{}({})'.format(class_name, child_class_name)

        # Create node's label.
        if show_node_names:
            label = '{}: {}'.format(node_name, class_name)
        else:
            label = class_name

        # Rebuild the label as a table including input/output shapes.
        if show_shapes:
            try:
                outputlabels = str(node.output_shape)
            except AttributeError:
                outputlabels = 'multiple'
            if hasattr(node, 'input_shape'):
                inputlabels = str(node.input_shape)
            elif hasattr(node, 'input_shapes'):
                inputlabels = ', '.join(
                    [str(ishape) for ishape in node.input_shapes])
            else:
                inputlabels = 'multiple'
            label = '%s\n|{input:|output:}|{{%s}|{%s}}' % (label,
                                                           inputlabels,
                                                           outputlabels)
        node = pydot.Node(node_id, label=label)
        dot.add_node(node)

    # Connect nodes with edges.
    for node in nodes:
        node_id = str(id(node))
        for i, inbound_node in enumerate(node._inbound_nodes):
            inbound_node_id = str(id(inbound_node))
            node_id = str(id(node))
            dot.add_edge(pydot.Edge(inbound_node_id, node_id))
    return dot


def plot_graph(graph,
               to_file='model.png',
               show_shapes=False,
               show_node_names=True):
    dot = graph_to_dot(graph, show_shapes, show_node_names)
    _, extension = os.path.splitext(to_file)
    if not extension:
        extension = 'png'
    else:
        extension = extension[1:]
    dot.write(to_file, format=extension)



from .base_node import Node


class Sequential(Node):
    def __init__(self, node=None, name=None):
        super(Sequential, self).__init__(name=name)

        self._nodes = []

        if node:
            self.add(node)

        self._nodes.reverse()

    def add(self, node):
        # print('add node: ' + str(node))
        if not isinstance(node, Node):
            raise TypeError('The added node must be '
                            'an instance of class Node. '
                            'Found: ' + str(node))

        self._nodes.append(node)
        for sub_node in node._inbound_nodes:
            self.add(sub_node)



from .engine.base_node import Node
from .engine.sequential import Sequential
import cv2 as cv


class Graph(Sequential):
    def __init__(self, node, name=None):
        if not isinstance(node, Node):
            raise TypeError('The added node must be '
                            'an instance of class Node. '
                            'Found: ' + str(node))
        super(Graph, self).__init__(node=node, name=name)
        self._output = node

    def run(self):
        source = self._nodes[0]

        while True:
            ret, frame = source.call()
            if ret:
                for node in self._nodes[1:]:
                    frame = node.call(frame)

                ch = cv.waitKey(1)
                if ch == 27:
                    break




import setuptools

setuptools.setup(
    name="gaze",
    version="0.0.2",
    author="Peisheng Wang",
    author_email="t-peiwa@microsoft.com",
    description="Gaze integrates the computing power of private clouds with data from IoT devices such as cameras,"
                " providing scalable video analytics capacity on the Edge",
    url="https://github.com/foamliu/Gaze",
    packages=setuptools.find_packages(),
    classifiers=(
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ),
)